#include "para_free_metric.h"

ParaFreeMetricMon::ParaFreeMetricMon()
{
  init();
}

ParaFreeMetricMon::~ParaFreeMetricMon()
{
  mem.bufferDestory();
  cpu.bufferDestory();
  disk.bufferDestory();
  net.bufferDestory();
  delete lastUser;
  delete lastNice;
  delete lastSys;
  delete lastIdle;
  delete lastIowait;
  delete lastHardirq;
  delete lastSoftirq;
  delete lastTotal;
  delete lastRsect;
  delete lastWsect;
  delete lastBytesOut;
  delete lastBytesIn;
}

void ParaFreeMetricMon::collector()
{
  char buffer[1024];
  char dev_name[256];
  FILE *fp;
  int i;
  int statistic;

  // mem
  float memTotal,memFree,buffers,cached,swapCached,
      swapTotal,swapFree,memRatio,swapRatio;
  if (NULL == (fp=fopen(MEMINFO,"r"))) error();
  statistic=0;
  while (NULL != fgets(buffer,sizeof(buffer),fp)) {
    if (!memcmp("MemTotal",buffer,8)) {
      sscanf(buffer,"MemTotal:%f kB",&memTotal);
      statistic++;
    } else if (!memcmp("MemFree",buffer,7)) {
      sscanf(buffer,"MemFree:%f",&memFree);
      statistic++;
    } else if (!memcmp("Buffers",buffer,7)) {
      sscanf(buffer,"Buffers:%f",&buffers);
      statistic++;
    } else if (!memcmp("Cached",buffer,6)) {
      sscanf(buffer,"Cached:%f",&cached);
      statistic++;
    } else if (!memcmp("SwapCached",buffer,10)) {
      sscanf(buffer,"SwapCached:%f",&swapCached);
      statistic++;
    } else if (!memcmp("SwapTotal",buffer,9)) {
      sscanf(buffer,"SwapTotal:%f",&swapTotal);
      statistic++;
    } else if(!memcmp("SwapFree",buffer,8)) {
      sscanf(buffer,"SwapFree:%f",&swapFree);
      statistic++;
    }
    if (7==statistic) break;
  }
  fclose(fp);
  memRatio = memTotal?(memTotal-memFree-buffers-cached)/memTotal*100:0;
  swapRatio = swapTotal?(swapTotal-swapFree)/swapTotal*100:0;
  mem.addEventVal(0,0,memTotal);
  mem.addEventVal(0,1,memFree);
  mem.addEventVal(0,2,buffers);
  mem.addEventVal(0,3,cached);
  mem.addEventVal(0,4,swapCached);
  mem.addEventVal(0,5,swapTotal);
  mem.addEventVal(0,6,swapFree);
  mem.addEventVal(0,7,memRatio);
  mem.addEventVal(0,8,swapRatio);
  
  // cpu
  int64_t user,nice,sys,idle,iowait,hardirq,softirq,total,totalD;
  int nCpu = get_nprocs();
  if (NULL == (fp=fopen(CPUINFO,"r"))) error();
  fgets(buffer,sizeof(buffer),fp);
  for (i = 0; i < nCpu; ++i) {
    fgets(buffer,sizeof(buffer),fp);
#if __WORDSIZE == 64
    sscanf(&buffer[5],"%ld %ld %ld %ld %ld %ld %ld",&user,&nice,&sys,&idle,&iowait,&hardirq,&softirq);
#else
    sscanf(&buffer[5],"%lld %lld %lld %lld %lld %lld %lld",&user,&nice,&sys,&idle,&iowait,&hardirq,&softirq);
#endif
    total = user+nice+sys+idle+iowait+hardirq+softirq;
    totalD = total-lastTotal[i];
    cpu.addEventVal(i,0,totalD?(float)(1-((double)(idle-lastIdle[i]))/totalD)*100:(float)0);
    cpu.addEventVal(i,1,totalD?(float)(((double)(user-lastUser[i]))/totalD*100):(float)0);
    cpu.addEventVal(i,2,totalD?(float)(((double)(nice-lastNice[i]))/totalD*100):(float)0);
    cpu.addEventVal(i,3,totalD?(float)(((double)(sys-lastSys[i]))/totalD*100):(float)0);
    cpu.addEventVal(i,4,totalD?(float)(((double)(idle-lastIdle[i]))/totalD*100):(float)100);
    cpu.addEventVal(i,5,totalD?(float)(((double)(iowait-lastIowait[i]))/totalD*100):(float)0);
    cpu.addEventVal(i,6,totalD?(float)(((double)(hardirq-lastHardirq[i]))/totalD*100):(float)0);
    cpu.addEventVal(i,7,totalD?(float)(((double)(softirq-lastSoftirq[i]))/totalD*100):(float)0);
    lastUser[i] = user;
    lastNice[i] = nice;
    lastSys[i] = sys;
    lastIdle[i] = idle;
    lastIowait[i] = iowait;
    lastHardirq[i] = hardirq;
    lastSoftirq[i] = softirq;
    lastTotal[i] = total;
  }
  fclose(fp);

  // disk
  int major;
  int64_t rsect,wsect;
  if (NULL == (fp=fopen(DISKINFO,"r"))) error();
  i = 0;
  while (NULL != fgets(buffer,sizeof(buffer),fp)) {
#if __WORDSIZE == 64
    sscanf(buffer,"%d %*d %s %*d %*d %ld %*d %*d %*d %ld",&major,dev_name,&rsect,&wsect);
#else
    sscanf(buffer,"%d %*d %s %*d %*d %lld %*d %*d %*d %lld",&major,dev_name,&rsect,&wsect);
#endif
    if (isDiskMonitored(major,dev_name)) {
      disk.addEventVal(i,0,(float)((double)(rsect-lastRsect[i]))*512);
      disk.addEventVal(i,1,(float)((double)(wsect-lastWsect[i]))*512);
      lastRsect[i] = rsect;
      lastWsect[i] = wsect;
      i++;
    }
  }
  fclose(fp);

  // net
  int64_t bytesOut,bytesIn;
  if (NULL == (fp=fopen(NETINFO,"r"))) error();
  fgets(buffer,sizeof(buffer),fp);
  fgets(buffer,sizeof(buffer),fp);
  i = 0;
  while (NULL != fgets(buffer,sizeof(buffer),fp)) {
    sscanf(buffer," \t%[^:]",dev_name);
    if (isNetMonitored(dev_name)) {
#if __WORDSIZE == 64
      sscanf(buffer,"%*[^:]:%ld%*d%*d%*d%*d%*d%*d%*d%ld",&bytesIn,&bytesOut);
#else
      sscanf(buffer,"%*[^:]:%lld%*d%*d%*d%*d%*d%*d%*d%lld",&bytesIn,&bytesOut);
#endif
      net.addEventVal(i,0,(float)((double)(bytesOut-lastBytesOut[i])));
      net.addEventVal(i,1,(float)((double)(bytesIn-lastBytesIn[i])));
      lastBytesOut[i] = bytesOut;
      lastBytesIn[i] = bytesIn;
      i++;
    }
  }
  fclose(fp);

}


void ParaFreeMetricMon::titlePrint()
{
  mem.titlePrint();
  cpu.titlePrint();
  disk.titlePrint();
  net.titlePrint();
}

void ParaFreeMetricMon::valuePrint()
{
  mem.valuePrint();
  cpu.valuePrint();
  disk.valuePrint();
  net.valuePrint();
}

ParaDeviceArray* ParaFreeMetricMon::getDevice(int d)
{
  if (0 == d)
    return &mem;
  else if (1 == d)
    return &cpu;
  else if (2 == d)
    return &disk;
  else if (3 == d)
    return &net;
  else
    return NULL;
}

void ParaFreeMetricMon::init()
{
  char buffer[1024];
  char dev_name[256];
  int major;
  FILE *fp;
  int i;

  // mem
  mem.setDeviceName("mem");
  mem.appendSector("mem");
  mem.appendEvent("memTotal");
  mem.appendEvent("memFree");
  mem.appendEvent("buffers");
  mem.appendEvent("cached");
  mem.appendEvent("swapCached");
  mem.appendEvent("swapTotal");
  mem.appendEvent("swapFree");
  mem.appendEvent("memRatio");
  mem.appendEvent("swapRatio");
  mem.bufferInit(); 
  
  // cpu
  cpu.setDeviceName("cpu");
  int nCpu = get_nprocs();
  char buf[256];
  for (i = 0; i < nCpu; ++i) {
    sprintf(buf,"cpu%d",i);
    cpu.appendSector(buf);
  }
  cpu.appendEvent("used");
  cpu.appendEvent("user");
  cpu.appendEvent("nice");
  cpu.appendEvent("sys");
  cpu.appendEvent("idle");
  cpu.appendEvent("iowait");
  cpu.appendEvent("hardirq");
  cpu.appendEvent("softirq");
  cpu.bufferInit();
  lastUser = new int64_t[nCpu];
  lastNice = new int64_t[nCpu];
  lastSys = new int64_t[nCpu];
  lastIdle = new int64_t[nCpu];
  lastIowait = new int64_t[nCpu];
  lastHardirq = new int64_t[nCpu];
  lastSoftirq = new int64_t[nCpu];
  lastTotal = new int64_t[nCpu];
  memset(lastUser,0,nCpu*sizeof(int64_t));
  memset(lastNice,0,nCpu*sizeof(int64_t));
  memset(lastSys,0,nCpu*sizeof(int64_t));
  memset(lastIdle,0,nCpu*sizeof(int64_t));
  memset(lastIowait,0,nCpu*sizeof(int64_t));
  memset(lastHardirq,0,nCpu*sizeof(int64_t));
  memset(lastSoftirq,0,nCpu*sizeof(int64_t));
  memset(lastTotal,0,nCpu*sizeof(int64_t));

  // disk
  disk.setDeviceName("disk");
  if (NULL == (fp=fopen(DISKINFO,"r"))) error();
  while (NULL != fgets(buffer,sizeof(buffer),fp)) {
    sscanf(buffer,"%d %*d %s",&major,dev_name);
    if (isDiskMonitored(major,dev_name)) {
      disk.appendSector(dev_name);
    }
  }
  fclose(fp);
  disk.appendEvent("read");
  disk.appendEvent("write");
  disk.bufferInit();
  lastRsect = new int64_t[disk.getNSectors()];
  lastWsect = new int64_t[disk.getNSectors()];
  memset(lastRsect,0,disk.getNSectors()*sizeof(int64_t));
  memset(lastWsect,0,disk.getNSectors()*sizeof(int64_t));

  // net
  net.setDeviceName("net");
  if (NULL == (fp=fopen(NETINFO,"r"))) error();
  fgets(buffer,sizeof(buffer),fp);
  fgets(buffer,sizeof(buffer),fp);
  while (NULL != fgets(buffer,sizeof(buffer),fp)) {
    sscanf(buffer," \t%[^:]",dev_name);
    if (isNetMonitored(dev_name)) {
      net.appendSector(dev_name);
    }
  }
  fclose(fp);
  net.appendEvent("out");
  net.appendEvent("in");
  net.bufferInit();
  lastBytesOut = new int64_t[net.getNSectors()];
  lastBytesIn = new int64_t[net.getNSectors()];
  memset(lastBytesOut,0,net.getNSectors()*sizeof(int64_t));
  memset(lastBytesIn,0,net.getNSectors()*sizeof(int64_t));
}

void ParaFreeMetricMon::error()
{
  printf("ParaFreeMetricMon ERROR!!!(Please reload)\n");
}

int ParaFreeMetricMon::isDiskMonitored(int major,char *name)
{
  char sysPath[PATH_MAX];
  char *slash;
  if (1 == major || 9 == major)
    return 0;
  while ((slash = strchr(name,'/')))
    *slash='!';
  snprintf(sysPath,sizeof(sysPath),"/sys/block/%s",name);
  return !(access(sysPath,F_OK));
}

int ParaFreeMetricMon::isNetMonitored(char *name)
{
  if (!strcmp("lo",name) || !strcmp("sit0",name))
    return 0;
  return 1;
}

