#include "para_graph.h"

ParaGraph::ParaGraph()
{
  setAutoFillBackground(true);
  QPalette palette;
  palette.setColor(backgroundRole(), GRAY_COLOR);
  setPalette(palette);
  ratio = 4;
  rateMax = 100.0;
  diskMax = 200.0;
  netMax = 100.0;
  helpStatus = true;
}

ParaGraph::ParaGraph(QWidget* parent):
        QWidget(parent)
{
  setAutoFillBackground(true);
  QPalette palette;
  palette.setColor(backgroundRole(),GRAY_COLOR);
  setPalette(palette);
  ratio = 4;
  rateMax = 100.0;
  diskMax = 200.0;
  netMax = 100.0;
  helpStatus = true;
}

ParaGraph::~ParaGraph()
{
  delete hostScale;
  delete hostWindow;
}

void ParaGraph::setHost()
{
  hostScale = new ParaHostScale();
  hostScale->setOrigPoint(0,0);
  x = hostScale->getBorderPixel(0).getX();
  y = hostScale->getBorderPixel(0).getY();
  this->resize(int(x*ratio),int(y*ratio));    //set unit window as variable size
  hostWindow = new ParaHostWindow();
  hostWindow->setRateMax(this->rateMax);
  hostWindow->setDiskMax(this->diskMax);
  hostWindow->setNetMax(this->netMax);
  hostWindow->setRatio(ratio);
  hostWindow->setFontRatio(ratio);
  hostWindow->setHostScale(hostScale);
  hostWindow->setHostWindow();
  hostWindow->help(true);
  helpStatus = true;
  curSize.setWidth(x*ratio);
  curSize.setHeight(y*ratio);
}

void ParaGraph::setRealModelInit(int n)
{
  QPalette palette;
  palette.setColor(backgroundRole(),GRAY_COLOR);
  setPalette(palette);
  hostWindow->setNMetrics(n);
}

QSizeF ParaGraph::getCurSize()
{
  return curSize;
}

float ParaGraph::getRatio()
{
  return ratio;
}

void ParaGraph::setRatio(float ratio1, float ratio2)
{
  hostWindow->setRatio(ratio1, ratio2);
}

void ParaGraph::setFontRatio(float ratio)
{
  hostWindow->setFontRatio(ratio);
}

float ParaGraph::getX()
{
  return x;
}

float ParaGraph::getY()
{
  return y;
}

QColor ParaGraph::getColor(PARA_WINDOW_AREA n)
{
  QColor c;
  c = hostWindow->getColor(n);
  return c;
}

void ParaGraph::realModel(float *display)
{
  hostWindow->setMetrics(display);
  this->update();
}

void ParaGraph::paintEvent(QPaintEvent *e)
{
  QWidget::paintEvent(e);
  QPainter painter(this);
  hostWindow->setHostVision(&painter);
}

void ParaGraph::resizeEvent(QResizeEvent *e)
{
  float wRatio, hRatio,fontRatio;
  wRatio = 4*e->size().width()/(float)(this->getCurSize().width());
  hRatio = 4*e->size().height()/(float)(this->getCurSize().height());
  fontRatio = wRatio<hRatio?wRatio:hRatio;
  if (wRatio > 4 && hRatio > 8) {
    fontRatio = wRatio;
  }
  this->setFontRatio(fontRatio);
  this->setRatio(wRatio,hRatio);
  this->resize(int(wRatio*(this->getX())), int(hRatio*(this->getY())));
}

void ParaGraph::keyPressEvent(QKeyEvent *e)
{
  const int v = 86;
  int keyCode = e->key();

  if (v == keyCode) {
    helpStatus = (helpStatus)?false:true;
    hostWindow->help(helpStatus);
  }
}
