#ifndef PARA_GRAPH_RES_H
#define PARA_GRAPH_RES_H

#include <QtGui>

#include "para_res.h"

#define DEFAULT_PEN_STYLE       Qt::SolidLine
#define DEFAULT_PEN_JOIN_STYLE  Qt::MiterJoin

#define PARA_GRAPH_VEC  int
#define PARA_UP         0
#define PARA_LEFT       1
#define PARA_DOWN       2
#define PARA_RIGHT      3

#define PARA_WINDOW_AREA  int
#define PARA_CPU          0
#define PARA_MEMORY       1
#define PARA_DISK_READ    2
#define PARA_DISK_WRITE   3
#define PARA_NET_OUT      4
#define PARA_NET_IN       5
#define PARA_SWAP         6
#define PARA_CPU_SY_WA    7

/*********************
  10------------11
  | 0----------1 |
  | |          | |
  | |          | |
  | |          | |
  | 3----------2 |
  13------------12
**********************/

class ParaHostScale
{
 public:
  ParaHostScale();
  ~ParaHostScale();
  
  void setOrigPoint(int x,int y);
  void setOrigPoint(ParaPoint &point);
  
  ParaPoint& getBorderPoint(int point);
  ParaPoint& getMemoryPoint(int point);
  ParaPoint& getSwapPoint(int point);
  ParaPoint& getCpuPoint(int point);
  ParaPoint& getDiskReadPoint(int point);
  ParaPoint& getDiskWritePoint(int point);
  ParaPoint& getNetOutPoint(int point);
  ParaPoint& getNetInPoint(int point);
  
  ParaPixel& getBorderPixel(int line);
  ParaPixel& getMemoryPixel(int line);
  ParaPixel& getSwapPixel(int line);
  ParaPixel& getCpuPixel(int line);
  ParaPixel& getDiskReadPixel(int line);
  ParaPixel& getDiskWritePixel(int line);
  ParaPixel& getNetOutPixel(int line);
  ParaPixel& getNetInPixel(int line);

 private:
  void initHostScale();

  ParaPoint orig;
  
  ParaPoint defPoint;
  ParaPoint borderStart;
  ParaPoint memoryStart,swapStart,cpuStart,
            diskReadStart,diskWriteStart,
            netOutStart,netInStart;
 
  ParaPixel defPixel;
  ParaPixel borderArea;
  ParaPixel memoryArea,swapArea,cpuArea,
            diskReadArea,diskWriteArea,
            netOutArea,netInArea;
};

#endif
