#include "para_host_window.h"

#define DISPLAY_FONT "Nimbus Mono L"

ParaHostWindow::ParaHostWindow()
{
  setAutoFillBackground(true);
  nSwapMetrics = nCpuMetrics = nDiskReadMetrics = nDiskWriteMetrics = nNetOutMetrics = nNetInMetrics = 0;
  cpuAllUsed = 0;
  memset(memoryMetrics,0,PARA_MAX_CPU);
  memset(swapMetrics,0,PARA_MAX_CPU);
  memset(cpuUsedMetrics,0,PARA_MAX_CPU);
  memset(cpuSyWaMetrics,0,PARA_MAX_CPU);
  memset(diskReadMetrics,0,PARA_MAX_CPU);
  memset(diskWriteMetrics,0,PARA_MAX_CPU);
  memset(netOutMetrics,0,PARA_MAX_CPU);
  memset(netInMetrics,0,PARA_MAX_CPU);
  memoryColor = MEMORY_COLOR;
  swapColor = SWAP_COLOR;
  cpuUsedColor = CPU_USED_COLOR;
  cpuSyWaColor = CPU_SY_WA_COLOR;
  diskReadColor = DISK_READ_COLOR;
  diskWriteColor = DISK_WRITE_COLOR;
  netOutColor = NET_OUT_COLOR;
  netInColor = NET_IN_COLOR;
  grayColor = GRAY_COLOR;
  defaultColor = DEFAULT_COLOR;

  isFirst = true;
  rateMax = 100.0;  
  diskMax = 100.0;
  netMax = 100.0;
  isColorFirst = true;
}

ParaHostWindow::~ParaHostWindow()
{

}

void ParaHostWindow::setRatio(float ratio1, float ratio2)
{
  this->ratio1 = ratio1;
  this->ratio2 = ratio2;
}

void ParaHostWindow::setRatio(float ratio)
{
  this->ratio1 = ratio;
  this->ratio2 = ratio;
}

void ParaHostWindow::setFontRatio(float ratio)
{
  this->fontRatio = ratio;
}

void ParaHostWindow::setHostScale(ParaHostScale *hostScale)
{
  this->hostScale = hostScale;
}

void ParaHostWindow::setHostWindow()
{
  setHostWeight();
}

void ParaHostWindow::setHostVision(QPainter *painter)
{
  this->painter = painter;
  setHostFill();
  setHelp();
}

void ParaHostWindow::setHelp()
{
  QFont font;

  font.setFamily(DISPLAY_FONT);
  font.setPointSize(10*fontRatio/4);
  font.setBold(true);
  painter->setFont(font);
  painter->setPen(QPen(Qt::white));

  if (true == helpStatus) {
    QString tmp;
    char tmpChar[32];
    
    sprintf(tmpChar,"%7.2f",cpuAllUsed);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("   0.00");
    }
    painter->drawText((int)(180*ratio1/4),(int)(42*ratio2/4),tr("CPU%1 %").arg(tmp));

    sprintf(tmpChar,"%3.2f",diskReadMetrics[0]);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("0.00");
    }
    painter->drawText((int)(55*ratio1/4),(int)(290*ratio2/4),tr("Disk Read"));
    painter->drawText((int)(55*ratio1/4),(int)(310*ratio2/4),tr("%1 MB/s").arg(tmp));

    sprintf(tmpChar,"%3.2f",diskWriteMetrics[0]);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("0.00");
    }
    painter->drawText((int)(220*ratio1/4),(int)(290*ratio2/4),tr("Disk Write"));
    painter->drawText((int)(220*ratio1/4),(int)(310*ratio2/4),tr("%1 MB/s").arg(tmp));

    sprintf(tmpChar,"%2.2f",swapMetrics[0]);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("0.00");
    }
    painter->drawText((int)(16*ratio1/4),(int)(30*ratio2/4),tr("Swap"));
    painter->drawText((int)(16*ratio1/4),(int)(47*ratio2/4),tr("%1%").arg(tmp));

    tmp = tr("%1").arg(memoryMetrics[0]);
    tmp = tmp.left(4);
    if (isFirst) {
      tmp = QString(" 0.0");
    }
    painter->drawText((int)(14*ratio1/4),(int)(120*ratio2/4),tr("Memory"));
    painter->drawText((int)(14*ratio1/4),(int)(140*ratio2/4),tr("%1%").arg(tmp));
    
    sprintf(tmpChar,"%3.2f",netOutMetrics[0]);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("0.00");
    }
    painter->drawText((int)(360*ratio1/4),(int)(70*ratio2/4),tr("Net Send"));
    painter->drawText((int)(360*ratio1/4),(int)(90*ratio2/4),tr("%1 MB/s").arg(tmp));
    
    sprintf(tmpChar,"%3.2f",netInMetrics[0]);
    tmp = QString(tmpChar);
    if (isFirst) {
      tmp = QString("0.00");
    }
    painter->drawText((int)(360*ratio1/4),(int)(220*ratio2/4),tr("Net Recv"));
    painter->drawText((int)(360*ratio1/4),(int)(240*ratio2/4),tr("%1 MB/s").arg(tmp));

    isFirst = false;
  }
}

void ParaHostWindow::help(bool status)
{
  helpStatus = status;
}

QColor ParaHostWindow::getColor(PARA_WINDOW_AREA n)
{
  if (PARA_CPU == n) {
    return cpuUsedColor;
  } else if (PARA_CPU_SY_WA == n) {
    return cpuSyWaColor;
  } else if (PARA_MEMORY == n) {
    return memoryColor;
  } else if (PARA_DISK_READ == n) {
    return diskReadColor;
  } else if (PARA_DISK_WRITE == n) {
    return diskWriteColor;
  } else if (PARA_NET_OUT == n) {
    return netOutColor ;
  } else if (PARA_NET_IN == n) {
    return netInColor ;
  } else if(PARA_SWAP == n) {
    return swapColor;
  }

  return QColor();
}

void ParaHostWindow::setNMetrics(int nDisplay)
{
  nMemoryMetrics = 1;
  nSwapMetrics = 1;
  nCpuMetrics = (nDisplay-6)/2;
  nDiskReadMetrics = 1;
  nDiskWriteMetrics = 1;
  nNetOutMetrics = 1;
  nNetInMetrics = 1;    
}


void ParaHostWindow::setMetrics(float *display)
{
  int i;
  memoryMetrics[0] = display[0];
  swapMetrics[0] = display[1];
  cpuAllUsed = 0;
  for (i = 0; i < nCpuMetrics; ++i) {
    cpuUsedMetrics[i] = display[6+i];
    cpuSyWaMetrics[i] = display[6+nCpuMetrics+i];
    cpuAllUsed += display[6+i];
  }
  cpuAllUsed = cpuAllUsed/nCpuMetrics;
  diskReadMetrics[0] = display[2];
  diskWriteMetrics[0] = display[3];
  netOutMetrics[0] = display[4];
  netInMetrics[0] = display[5];
}

void ParaHostWindow::setHostWeight()
{
  memoryWeight = hostScale->getMemoryPixel(0).getY()/rateMax;
  swapWeight = hostScale->getSwapPixel(0).getY()/rateMax;
  cpuWeight = hostScale->getCpuPixel(0).getY()/rateMax;
  diskReadWeight = hostScale->getDiskReadPixel(0).getX()/diskMax;
  diskWriteWeight = hostScale->getDiskWritePixel(0).getX()/diskMax;
  netOutWeight = hostScale->getNetOutPixel(0).getY()/netMax;
  netInWeight = hostScale->getNetInPixel(0).getY()/netMax;
}

void ParaHostWindow::setHostFill()
{
  setMemoryFill();
  setSwapFill();
  setCpuFill();
  setDiskReadFill();
  setDiskWriteFill();
  setNetOutFill();
  setNetInFill();
}

void ParaHostWindow::setMemoryFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nMemory;

  x = hostScale->getMemoryPoint(0).getX();
  y = hostScale->getMemoryPoint(0).getY();
  size = hostScale->getMemoryPixel(0).getX();
  
  nMemory = nMemoryMetrics;
  loop = nMemory>64?64:nMemory;
  offStart = 0;
  perSplit = (nMemory==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;

  for (int i = 0; i < loop; ++i) {
    setMemoryFillSingle(start+(perSize+perSplit)*i,y,perSize,memoryMetrics[i]*memoryWeight);
  }
}

void ParaHostWindow::setSwapFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nSwap;
  
  x = hostScale->getSwapPoint(0).getX();
  y = hostScale->getSwapPoint(0).getY();
  size = hostScale->getSwapPixel(0).getX();

  nSwap = nSwapMetrics;
  loop = nSwap>64?64:nSwap;
  offStart = 0;
  perSplit = (nSwap==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;

  for (int i = 0; i < loop; ++i) {
    setSwapFillSingle(start+(perSize+perSplit)*i,y,perSize,swapMetrics[i]*swapWeight);
  }
}

void ParaHostWindow::setCpuFill()
{
  float x,y,size;
  float start,offStart,perSplit,perSize;
  int loop,ncpu;

  x = hostScale->getCpuPoint(0).getX();
  y = hostScale->getCpuPoint(0).getY();
  size = hostScale->getCpuPixel(0).getX();
  
  ncpu = nCpuMetrics;
  loop = ncpu>64?64:ncpu;
  offStart = 0;
  perSplit = (ncpu==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;
  
  for (int i = 0; i < loop; ++i) {
    setCpuFillSingle(start+(perSize+perSplit)*i,y,perSize,cpuUsedMetrics[i]*cpuWeight,cpuSyWaMetrics[i]*cpuWeight);
  }
}

void ParaHostWindow::setDiskReadFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nDiskRead;

  x = hostScale->getDiskReadPoint(0).getX();
  y = hostScale->getDiskReadPoint(0).getY();
  size = hostScale->getDiskReadPixel(0).getY();
  
  nDiskRead = nDiskReadMetrics;
  loop = nDiskRead>64?64:nDiskRead;
  offStart = 0;
  perSplit = (nDiskRead==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;

  for (int i = 0; i < loop; ++i) {
    setDiskReadFillSingle(start,y+(perSize+perSplit)*i,perSize,diskReadMetrics[i]*diskReadWeight);
  }
}

void ParaHostWindow::setDiskWriteFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nDiskWrite;

  x = hostScale->getDiskWritePoint(0).getX();
  y = hostScale->getDiskWritePoint(0).getY();
  size = hostScale->getDiskWritePixel(0).getY();

  nDiskWrite = nDiskWriteMetrics;
  loop = nDiskWrite>64?64:nDiskWrite;
  offStart = 0;
  perSplit = (nDiskWrite==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;

  for(int i = 0; i < loop; ++i)
  {
    setDiskWriteFillSingle(start,y+(perSize+perSplit)*i,perSize,diskWriteMetrics[i]*diskWriteWeight);
  }
}

void ParaHostWindow::setNetOutFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nNetOut;

  x = hostScale->getNetOutPoint(0).getX();
  y = hostScale->getNetOutPoint(0).getY();
  size = hostScale->getNetOutPixel(0).getX();
     
  nNetOut = nNetOutMetrics;
  loop = nNetOut>64?64:nNetOut;
  offStart = 0;
  perSplit = (nNetOut==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;

  for (int i = 0; i < loop; ++i) {
    setNetOutFillSingle(start+(perSize+perSplit)*i,y,perSize,netOutMetrics[i]*netOutWeight);
  }
}

void ParaHostWindow::setNetInFill()
{
  float x,y,size;
  float start,offStart,perSize,perSplit;
  int loop,nNetIn;

  x = hostScale->getNetInPoint(0).getX();
  y = hostScale->getNetInPoint(0).getY();
  size = hostScale->getNetInPixel(0).getX();
  
  nNetIn = nNetInMetrics;
  loop = nNetIn>64?64:nNetIn;
  offStart = 0;
  perSplit = (nNetIn==1?0:1)-(loop>12?0.5:0);
  perSize = (size-perSplit*(loop-1))/loop;
  start = offStart+x;
  
  for (int i = 0; i < loop; ++i) {
    setNetInFillSingle(start+(perSize+perSplit)*i,y,perSize,netInMetrics[i]*netInWeight);
  }
}

void ParaHostWindow::setMemoryFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;

  limit = hostScale->getMemoryPixel(0).getY();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getMemoryPixel(0).getY()-1;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }
  solidSize = size-1;
  start = x;
  base = y;
  rect1.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val1*ratio2);
  rect2.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val2*ratio2);
  paintGraph(rect2,defaultColor,PARA_UP);
  paintGraph(rect1,memoryColor,PARA_UP);
}

void ParaHostWindow::setSwapFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;
  
  limit = hostScale->getSwapPixel(0).getY();
  val1 = used>limit?limit:used;
  val2 = hostScale->getSwapPixel(0).getY();
  solidSize = size-1;
  start = x;
  base = y;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }
  rect1.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val1*ratio2);
  rect2.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val2*ratio2);

  paintGraph(rect2,defaultColor,PARA_UP);
  paintGraph(rect1,swapColor,PARA_UP);
}

void ParaHostWindow::setCpuFillSingle(float x,float y,float size,float used,float sy_wa)
{
  float limit;
  float val1,val2,val3;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2,rect3;

  limit = hostScale->getCpuPixel(0).getY();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getCpuPixel(0).getY()-1;
  val3 = sy_wa>limit?limit-1:sy_wa-1;

  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }
  if (isinf(val3) == 1) {
    val3 = 0;
  } else if (isnan(val3) != 0) {
    val3 = 0;
  }
  solidSize = size-1;
  start = x;
  base = y; 

  rect1.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val1*ratio2);
  rect2.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val2*ratio2);
  rect3.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val3*ratio2);
  
  paintGraph(rect2,defaultColor,PARA_UP);
  paintGraph(rect1,cpuUsedColor,PARA_UP);
  paintGraph(rect3,cpuSyWaColor,PARA_UP);
}

void ParaHostWindow::setDiskReadFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;

  limit = hostScale->getDiskReadPixel(0).getX();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getDiskReadPixel(0).getX()-1;
  solidSize = size-1;
  start = y;
  base = x;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }

  rect1.setRect(base*ratio1,start*ratio2,-val1*ratio1,solidSize*ratio2);
  rect2.setRect(base*ratio1,start*ratio2,-val2*ratio1,solidSize*ratio2);
  paintGraph(rect2,defaultColor,PARA_RIGHT);
  paintGraph(rect1,diskReadColor,PARA_RIGHT);

}

void ParaHostWindow::setDiskWriteFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;
  
  limit = hostScale->getDiskWritePixel(0).getX();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getDiskWritePixel(0).getX()-1;
  solidSize = size-1;
  start = y;
  base = x;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }

  rect1.setRect(base*ratio1,start*ratio2,val1*ratio1,solidSize*ratio2);
  rect2.setRect(base*ratio1,start*ratio2,val2*ratio1,solidSize*ratio2);
  paintGraph(rect2,defaultColor,PARA_LEFT);
  paintGraph(rect1,diskWriteColor,PARA_LEFT);
}

void ParaHostWindow::setNetOutFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;

  limit = hostScale->getNetOutPixel(0).getY();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getNetOutPixel(0).getY()-1;
  solidSize = size-1;
  start = x;
  base = y;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
  } else if (isnan(val2) != 0) {
    val2 = 0;
  }

  rect1.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val1*ratio2);
  rect2.setRect(start*ratio1,base*ratio2,solidSize*ratio1,-val2*ratio2);
  paintGraph(rect2,defaultColor,PARA_UP);
  paintGraph(rect1,netOutColor,PARA_UP);
}

void ParaHostWindow::setNetInFillSingle(float x,float y,float size,float used)
{
  float limit;
  float val1,val2;
  float solidSize;
  float start;
  float base;
  QRectF rect1,rect2;
  
  limit = hostScale->getNetInPixel(0).getY();
  val1 = used>limit?limit-1:used-1;
  val2 = hostScale->getNetInPixel(0).getY()-1;
  solidSize = size-1;
  start = x;
  base = y;
  if (isinf(val1) == 1) {
    val1 = 0;
  } else if (isnan(val1) != 0) {
    val1 = 0;
  }
  if (isinf(val2) == 1) {
    val2 = 0;
   }else if (isnan(val2) != 0) {
    val2 = 0;
  }

  rect1.setRect(start*ratio1,base*ratio2,solidSize*ratio1,val1*ratio2);
  rect2.setRect(start*ratio1,base*ratio2,solidSize*ratio1,val2*ratio2);
  paintGraph(rect2,defaultColor,PARA_DOWN);
  paintGraph(rect1,netInColor,PARA_DOWN);
}

void ParaHostWindow::paintGraph(QRectF &rect,QColor color,PARA_GRAPH_VEC vec)
{
  QPen pen;
  qreal x,y,width,height;

  pen.setStyle(DEFAULT_PEN_STYLE);
  pen.setJoinStyle(DEFAULT_PEN_JOIN_STYLE);
  pen.setWidthF(ratio1);
  pen.setColor(color);
  
  painter->setRenderHint(QPainter::Antialiasing,true);
  painter->setPen(pen);
  painter->setBrush(color);

  rect.getRect(&x,&y,&width,&height);
  if ((width <= 0.999999/ratio1 && width >= -0.999999/ratio1) || 
      (height <= 0.99999/ratio2 && height >= -0.99999/ratio2)) {
    if (PARA_UP == vec) {
      if (height < (-0.5)) painter->drawLine(QLineF(x,y,x+width,y));
    } else if (PARA_LEFT == vec) {
      if (width > 0.5) painter->drawLine(QLineF(x,y,x,y+height));
    } else if (PARA_DOWN == vec) {
      if (height > 0.5) painter->drawLine(QLineF(x,y,x+width,y));
    } else if (PARA_RIGHT == vec) {
      if (width < (-0.5)) painter->drawLine(QLineF(x,y,x,y+height));
    } else {
      ;
    }
  } else {
    if (PARA_UP == vec) {
      if (height < 0) painter->drawRect(rect);
    } else if (PARA_LEFT == vec) {
      if (width > 0) painter->drawRect(rect);
    } else if (PARA_DOWN == vec) {
      if (height > 0) painter->drawRect(rect);
    } else if (PARA_RIGHT == vec) {
      if (width < 0) painter->drawRect(rect);
    } else {
      ;
    }
  }
}
