#ifndef PARA_HOST_WINDOW_H
#define PARA_HOST_WINDOW_H

#include <QWidget>
#include <QPainter>

#include "include.h"
#include "para_graph_res.h"

#define PARA_MAX_CPU  512

class ParaHostWindow:public QWidget
{
  Q_OBJECT
 public:
  ParaHostWindow();
  ~ParaHostWindow();

  void setRatio(float,float);
  void setRatio(float);
  void setFontRatio(float);
  void setHostScale(ParaHostScale *hostScale);
  void setHostWindow();                         // set the layout of the host window
  void setHostVision(QPainter *painter);        // set the host window to show information
  void setHelp();                               // show detailed performance data
  void help(bool status);                       // set whether to show detailed performance data
  QColor getColor(PARA_WINDOW_AREA);
  void setRateMax(float data){this->rateMax = data;}
  void setDiskMax(float data){this->diskMax = data;}
  void setNetMax(float data){this->netMax = data;}
 
 public slots:
  void setNMetrics(int nDisplay);
  void setMetrics(float *display);
 
 private:
  void setHostWeight();
  void setHostFill();       //draw histogram for each region
  void setMemoryFill();
  void setSwapFill();
  void setCpuFill();
  void setDiskReadFill();
  void setDiskWriteFill();
  void setNetOutFill();
  void setNetInFill();
  void setMemoryFillSingle(float x,float y,float size,float used);
  void setSwapFillSingle(float x,float y,float size,float used);
  void setCpuFillSingle(float x,float y,float size,float used,float sy_wa);
  void setDiskReadFillSingle(float x,float y,float size,float used);
  void setDiskWriteFillSingle(float x,float y,float size,float used);
  void setNetOutFillSingle(float x,float y,float size,float used);
  void setNetInFillSingle(float x,float y,float size,float used);
  void paintGraph(QRectF &rect,QColor color,PARA_GRAPH_VEC vec);

  float ratio1,ratio2;
  float fontRatio;
  float memoryWeight,swapWeight,cpuWeight,
        diskReadWeight,diskWriteWeight,
        netOutWeight,netInWeight;
  int nMemoryMetrics,nSwapMetrics,nCpuMetrics,
      nDiskReadMetrics,nDiskWriteMetrics,
      nNetOutMetrics,nNetInMetrics;

  float memoryMetrics[PARA_MAX_CPU],swapMetrics[PARA_MAX_CPU],
        cpuUsedMetrics[PARA_MAX_CPU],cpuSyWaMetrics[PARA_MAX_CPU],
        diskReadMetrics[PARA_MAX_CPU],diskWriteMetrics[PARA_MAX_CPU],
        netOutMetrics[PARA_MAX_CPU],netInMetrics[PARA_MAX_CPU];
  float cpuAllUsed;
  float rateMax,diskMax,netMax; // memory, swap and cpu are all specific values except for disk and net both absolute values

  QPainter *painter;  
  ParaHostScale *hostScale;

  QColor memoryColor;
  QColor swapColor;
  QColor cpuUsedColor;
  QColor cpuSyWaColor;
  QColor diskReadColor;
  QColor diskWriteColor;
  QColor netOutColor;
  QColor netInColor;
  QColor grayColor;
  QColor defaultColor;
  
  bool helpStatus;
  bool isFirst;
  bool isColorFirst;
};
#endif
