#include "para_metric.h"

ParaData::ParaData()
{
  init();
}

ParaData::~ParaData()
{
  delete metric;
  delete toDisplayBuf;
}

void ParaData::start()
{
  metric->start();
  connect(metric,SIGNAL(displayGet()),this,SLOT(windowsDisplay()),Qt::DirectConnection);
}

void ParaData::wait()
{
  metric->wait();
}

void ParaData::displayBuild()
{
  int i,n;

  n = mem->getNEvents();
  toDisplayBuf[0] = mem->getVEvents(0,n-2);
  toDisplayBuf[1] = mem->getVEvents(0,n-1);
  toDisplayBuf[2] = 0.0;
  toDisplayBuf[3] = 0.0;
  n = disk->getNSectors();
  for (i = 0; i < n; ++i) {
    toDisplayBuf[2] += disk->getVEvents(i,0)/1048576;
    toDisplayBuf[3] += disk->getVEvents(i,1)/1048576;
  }
  toDisplayBuf[4] = 0.0;
  toDisplayBuf[5] = 0.0;
  n = net->getNSectors();
  for (i = 0; i < n; ++i) {
    toDisplayBuf[4] += net->getVEvents(i,0)/1048576;
    toDisplayBuf[5] += net->getVEvents(i,1)/1048576;
  }
  n = cpu->getNSectors();
  for (i = 0; i < n; ++i) {
    toDisplayBuf[6+i] = cpu->getVEvents(i,0);
    toDisplayBuf[6+n+i] = cpu->getVEvents(i,3)+cpu->getVEvents(i,5);
  }
}

void ParaData::print()
{
  int i,j,ns,ne;

  // mem
  ns = mem->getNSectors();
  ne = mem->getNEvents();
  printf("DEVICE:%s(sectors:%d event:%d)\n",mem->getDeviceName(),mem->getNSectors(),mem->getNEvents());
  for (i = 0; i < ns; ++i) {
    printf("  %s\n",mem->getTSectors(i));
    for (j = 0; j < ne; ++j)
      printf("    %s\t\t%.2f\n",mem->getTEvents(j),mem->getVEvents(i,j));
  }

  // cpu
  ns = cpu->getNSectors();
  ne = cpu->getNEvents();
  printf("DEVICE:%s(sectors:%d event:%d)\n",cpu->getDeviceName(),cpu->getNSectors(),cpu->getNEvents());
  for (i = 0; i < ns; ++i) {
    printf("  %s\n",cpu->getTSectors(i));
    for (j = 0; j < ne; ++j)
      printf("    %s\t\t%.2f\n",cpu->getTEvents(j),cpu->getVEvents(i,j));
  }

  // disk
  ns = disk->getNSectors();
  ne = disk->getNEvents();
  printf("DEVICE:%s(sectors:%d event:%d)\n",disk->getDeviceName(),disk->getNSectors(),disk->getNEvents());
  for (i = 0; i < ns; ++i) {
    printf("  %s\n",disk->getTSectors(i));
    for (j = 0; j < ne; ++j)
      printf("    %s\t\t%.2f\n",disk->getTEvents(j),disk->getVEvents(i,j));
  }

  // net
  ns = net->getNSectors();
  ne = net->getNEvents();
  printf("DEVICE:%s(sectors:%d event:%d)\n",net->getDeviceName(),net->getNSectors(),net->getNEvents());
  for (i = 0; i < ns; ++i) {
    printf("  %s\n",net->getTSectors(i));
    for (j = 0; j < ne; ++j)
      printf("    %s\t\t%.2f\n",net->getTEvents(j),net->getVEvents(i,j));
  }
}

void ParaData::windowsDisplay()
{
  displayBuild();
  emit displaySend(toDisplayBuf);
}

void ParaData::init()
{
  metric = new ParaMetric;
  mem  = metric->getDevice(0);
  cpu  = metric->getDevice(1);
  disk = metric->getDevice(2);
  net  = metric->getDevice(3);
  displayInit();
}

void ParaData::displayInit()
{
  nToDisplayBuf = 6+2*cpu->getNSectors();
  toDisplayBuf = new float[nToDisplayBuf];
}
